/*
 * Copyright (C) 2017  Advanced Micro Devices, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
 * AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
#ifndef _mp_10_0_DEFAULT_HEADER
#define _mp_10_0_DEFAULT_HEADER


// addressBlock: mp_SmuMp0_SmnDec
#define mmMP0_SMN_C2PMSG_32_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_33_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_34_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_35_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_36_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_37_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_38_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_39_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_40_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_41_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_42_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_43_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_44_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_45_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_46_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_47_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_48_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_49_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_50_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_51_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_52_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_53_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_54_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_55_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_56_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_57_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_58_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_59_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_60_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_61_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_62_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_63_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_64_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_65_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_66_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_67_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_68_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_69_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_70_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_71_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_72_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_73_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_74_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_75_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_76_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_77_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_78_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_79_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_80_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_81_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_82_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_83_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_84_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_85_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_86_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_87_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_88_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_89_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_90_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_91_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_92_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_93_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_94_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_95_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_96_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_97_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_98_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_99_DEFAULT                                              0x00000000
#define mmMP0_SMN_C2PMSG_100_DEFAULT                                             0x00000000
#define mmMP0_SMN_C2PMSG_101_DEFAULT                                             0x00000000
#define mmMP0_SMN_C2PMSG_102_DEFAULT                                             0x00000000
#define mmMP0_SMN_C2PMSG_103_DEFAULT                                             0x00000000
#define mmMP0_SMN_IH_CREDIT_DEFAULT                                              0x00000000
#define mmMP0_SMN_IH_SW_INT_DEFAULT                                              0x00000000
#define mmMP0_SMN_IH_SW_INT_CTRL_DEFAULT                                         0x00000000


// addressBlock: mp_SmuMp1_SmnDec
#define mmMP1_SMN_C2PMSG_32_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_33_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_34_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_35_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_36_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_37_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_38_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_39_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_40_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_41_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_42_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_43_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_44_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_45_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_46_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_47_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_48_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_49_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_50_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_51_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_52_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_53_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_54_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_55_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_56_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_57_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_58_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_59_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_60_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_61_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_62_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_63_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_64_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_65_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_66_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_67_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_68_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_69_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_70_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_71_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_72_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_73_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_74_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_75_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_76_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_77_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_78_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_79_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_80_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_81_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_82_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_83_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_84_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_85_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_86_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_87_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_88_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_89_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_90_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_91_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_92_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_93_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_94_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_95_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_96_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_97_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_98_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_99_DEFAULT                                              0x00000000
#define mmMP1_SMN_C2PMSG_100_DEFAULT                                             0x00000000
#define mmMP1_SMN_C2PMSG_101_DEFAULT                                             0x00000000
#define mmMP1_SMN_C2PMSG_102_DEFAULT                                             0x00000000
#define mmMP1_SMN_C2PMSG_103_DEFAULT                                             0x00000000
#define mmMP1_SMN_IH_CREDIT_DEFAULT                                              0x00000000
#define mmMP1_SMN_IH_SW_INT_DEFAULT                                              0x00000000
#define mmMP1_SMN_IH_SW_INT_CTRL_DEFAULT                                         0x00000000
#define mmMP1_SMN_FPS_CNT_DEFAULT                                                0x00000000


#endif
